from typing import List

from bluer_options.terminal import show_usage


def help_validate(
    tokens: List[str],
    mono: bool,
) -> str:
    return show_usage(
        [
            "@sbc",
            "scroll_phat_hd",
            "validate",
        ],
        "validate scroll_phat_hd.",
        mono=mono,
    )


help_functions = {
    "validate": help_validate,
}



from bluer_objects import README

from bluer_sbc.designs.consts import assets2


image_template = assets2 + "ultrasonic-sensor-tester/{}?raw=true"

marquee = README.Items(
    [
        {
            "name": "ultrasonic-sensor-tester",
            "marquee": image_template.format("00.jpg"),
            "url": "./bluer_sbc/docs/ultrasonic-sensor-tester.md",
        }
    ]
)

items = README.Items(
    [
        {
            "marquee": image_template.format(f"{index:02}.jpg"),
            "name": "",
        }
        for index in range(6)
    ]
)



from typing import Tuple, List
import numpy as np

from blueness import module

from bluer_sbc import NAME

NAME = module.name(__file__, NAME)


class Imager:
    def capture(self) -> Tuple[bool, np.ndarray]:
        return True, np.zeros(())


class TemplateImager(Imager):
    def capture(self) -> Tuple[bool, np.ndarray]:
        success, image = super().capture()

        # TODO: capture the image here

        return success, image

    def signature(self) -> List[str]:
        return [self.__class__.__name__]



from typing import Tuple, Union, List
import numpy as np

from bluer_options.logger import crash_report

from bluer_sbc.imager.camera.generic import Camera
from bluer_sbc import env


class RPI_64_bit_Camera(Camera):
    def capture_function(self) -> Tuple[bool, np.ndarray]:
        success = False
        image = np.ones((1, 1, 3), dtype=np.uint8) * 127

        try:
            image = self.device.capture_array()
            success = True
        except Exception as e:
            crash_report(e)

        return success, image

    def close_function(self):
        self.device.stop()
        del self.device

    def get_resolution_function(self) -> Tuple[int, int]:
        width, height = self.device.stream_configuration("main")["size"]
        return [height, width]

    def open_function(
        self,
        resolution=Union[List[int], None],
    ):
        from picamera2 import Picamera2

        self.device = Picamera2()

        config = self.device.create_still_configuration(
            main={
                "size": (
                    env.BLUER_SBC_CAMERA_WIDTH,
                    env.BLUER_SBC_CAMERA_HEIGHT,
                )
            }
        )
        self.device.configure(config)

        self.device.start()



import argparse
import cv2
from datetime import datetime
import numpy as np
import os.path
import time


def capture(flip_v=True, device="/dev/spidev0.0"):
    from pylepton import Lepton

    with Lepton(device) as l:
        a, _ = l.capture()

    if flip_v:
        cv2.flip(a, 0, a)

    cv2.normalize(a, a, 0, 65535, cv2.NORM_MINMAX)
    np.right_shift(a, 8, a)

    return np.uint8(a)


def capture_and_save(path):
    try:
        for _ in range(2):
            image = capture(False, "/dev/spidev0.0")
            time.sleep(1)

        cv2.imwrite(os.path.join(path, "image_raw.jpg"), image)

        cv2.imwrite(
            os.path.join(path, "image.jpg"),
            cv2.resize(image, (1280, 960), interpolation=cv2.INTER_NEAREST),
        )

        print(
            "lepton.capture_and_save({}) completed: {}.".format(
                path, "x".join([str(dim) for dim in image.shape])
            )
        )
    except Exception as e:
        print("lepton.capture_and_save() crashed: {}".format(e))
        return False

    return True


if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "task",
        type=str,
        default="",
        help="capture",
    )
    parser.add_argument(
        "--output_path",
        default="",
    )
    args = parser.parse_args()

    success = False
    if args.task == "capture":
        success = capture_and_save(args.output_path)
    else:
        print("-lepton: {}: command not found.".format(args.task))

    if not success:
        print("-lepton: {}: failed.".format(args.task))



import os
from typing import List

from bluer_options.help.functions import get_help
from bluer_objects import file, README

from bluer_sbc import NAME, VERSION, ICON, REPO_NAME
from bluer_sbc.designs.cheshmak import items as cheshmak_items
from bluer_sbc.designs.cheshmak import marquee as cheshmak_marquee
from bluer_sbc.designs.blue_bracket import items as blue_bracket_items
from bluer_sbc.designs.bluer_swallow import items as bluer_swallow_items
from bluer_sbc.designs.bluer_swallow import marquee as bluer_swallow_marquee
from bluer_sbc.designs.bluer_swallow_head import items as bluer_swallow_head_items
from bluer_sbc.designs.bluer_swallow_head import marquee as bluer_swallow_head_marquee
from bluer_sbc.designs.bryce import items as bryce_items
from bluer_sbc.designs.bryce import marquee as bryce_marquee
from bluer_sbc.help.functions import help_functions

from bluer_sbc.designs.ultrasonic_sensor_tester import (
    marquee as ultrasonic_sensor_tester_marquee,
)
from bluer_sbc.designs.ultrasonic_sensor_tester import (
    items as ultrasonic_sensor_tester_items,
)


def build():
    return all(
        README.build(
            items=readme.get("items", []),
            cols=readme.get("cols", 3),
            path=os.path.join(file.path(__file__), readme["path"]),
            ICON=ICON,
            NAME=NAME,
            VERSION=VERSION,
            REPO_NAME=REPO_NAME,
            help_function=lambda tokens: get_help(
                tokens,
                help_functions,
                mono=True,
            ),
        )
        for readme in [
            {
                "items": bluer_swallow_head_marquee
                + ultrasonic_sensor_tester_marquee
                + cheshmak_marquee
                + bluer_swallow_marquee
                + bryce_marquee
                + blue_bracket_items,
                "path": "..",
            },
            {
                "items": cheshmak_items,
                "path": "./docs/cheshmak.md",
            },
            {
                "items": bluer_swallow_items,
                "path": "./docs/bluer-swallow.md",
            },
            {
                "items": bluer_swallow_head_items,
                "path": "./docs/bluer-swallow-head.md",
            },
            {
                "items": bryce_items,
                "path": "./docs/bryce.md",
            },
            {
                "items": ultrasonic_sensor_tester_items,
                "path": "./docs/ultrasonic-sensor-tester.md",
            },
        ]
        + [
            {"path": f"./docs/aliases/{item}.md"}
            for item in [
                "camera",
                "hardware",
                "rpi",
            ]
        ]
    )



from blueness import module
from bluer_options.help.functions import help_main

from bluer_sbc import NAME
from bluer_sbc.help.functions import help_functions

NAME = module.name(__file__, NAME)


help_main(NAME, help_functions)



from typing import List

from bluer_options.terminal import show_usage


def help_capture_image(
    tokens: List[str],
    mono: bool,
) -> str:
    return show_usage(
        [
            "@camera",
            "capture",
            "image",
        ],
        "capture an image.",
        mono=mono,
    )


def help_capture_video(
    tokens: List[str],
    mono: bool,
) -> str:
    args = [
        "[--length 10]",
        "[--preview 1]",
    ]

    return show_usage(
        [
            "@camera",
            "capture",
            "video",
        ]
        + args,
        "capture a video",
        mono=mono,
    )


def help_preview(
    tokens: List[str],
    mono: bool,
) -> str:
    args = [
        "[--length 10]",
    ]

    return show_usage(
        [
            "@camera",
            "preview",
            "[-]",
        ]
        + args,
        "preview.",
        mono=mono,
    )


help_functions = {
    "capture": {
        "image": help_capture_image,
        "video": help_capture_video,
    },
    "preview": help_preview,
}



from typing import List

from bluer_options.terminal import show_usage
from bluer_ai.help.generic import help_functions as generic_help_functions

from bluer_sbc.help.adafruit_rgb_matrix import (
    help_functions as help_adafruit_rgb_matrix,
)
from bluer_sbc.help.camera import help_functions as help_camera
from bluer_sbc.help.grove import help_functions as help_grove
from bluer_sbc.help.hat import help_functions as help_hat
from bluer_sbc.help.lepton import help_functions as help_lepton
from bluer_sbc.help.rpi import help_functions as help_rpi
from bluer_sbc.help.scroll_phat_hd import help_functions as help_scroll_phat_hd
from bluer_sbc.help.sparkfun_top_phat import help_functions as help_sparkfun_top_phat
from bluer_sbc.help.unicorn_16x16 import help_functions as help_unicorn_16x16
from bluer_sbc import ALIAS


help_functions = generic_help_functions(plugin_name=ALIAS)

help_functions.update(
    {
        "adafruit_rgb_matrix": help_adafruit_rgb_matrix,
        "camera": help_camera,
        "grove": help_grove,
        "hat": help_hat,
        "lepton": help_lepton,
        "rpi": help_rpi,
        "scroll_phat_hd": help_scroll_phat_hd,
        "sparkfun_top_phat": help_sparkfun_top_phat,
        "unicorn_16x16": help_unicorn_16x16,
    }
)



from bluer_options import host

if host.is_rpi():
    if host.is_64bit():
        from bluer_sbc.imager.camera.rpi_64_bit import RPI_64_bit_Camera as Camera
    else:
        from bluer_sbc.imager.camera.rpi import RPI_Camera as Camera
else:
    from bluer_sbc.imager.camera.generic import Camera

instance = Camera()



from bluer_objects import README

from bluer_sbc.designs.consts import assets2


image_template = assets2 + "bluer-swallow/design/head-v1/{}?raw=true"

marquee = README.Items(
    [
        {
            "name": "bluer-swallow head",
            "marquee": image_template.format("01.jpg"),
            "url": "./bluer_sbc/docs/bluer-swallow-head.md",
        }
    ]
)

items = README.Items(
    [
        {
            "marquee": image_template.format(f"{index+1:02}.jpg"),
            "name": "",
        }
        for index in range(6)
    ]
)



from typing import Tuple
import numpy as np

from blueness import module
from bluer_options import string
from bluer_objects import file
from bluer_objects import path
from bluer_objects.env import abcli_path_git
from bluer_objects.host import shell

from bluer_sbc import NAME
from bluer_sbc.imager.classes import Imager
from bluer_sbc.logger import logger


NAME = module.name(__file__, NAME)


class Lepton(Imager):
    def capture(self) -> Tuple[bool, np.ndarray]:
        success, image = super().capture()

        temp_dir = path.auxiliary("lepton")
        success = shell(
            f"python python2.py capture --output_path {temp_dir}",
            work_dir=f"{abcli_path_git}/bluer-sbc/bluer_sbc/imager/lepton",
        )

        if success:
            success, image = file.load_image(f"{temp_dir}/image.jpg")

        if success:
            logger.info(f"{NAME}.capture(): {string.pretty_shape_of_matrix(image)}")

        return success, image



from typing import List

from bluer_options.terminal import show_usage, xtra


def help_capture(
    tokens: List[str],
    mono: bool,
) -> str:
    return show_usage(
        [
            "@sbc",
            "lepton",
            "capture",
        ],
        "lepton.capture.",
        mono=mono,
    )


def help_preview(
    tokens: List[str],
    mono: bool,
) -> str:
    return show_usage(
        [
            "@sbc",
            "lepton",
            "preview",
        ],
        "lepton.preview.",
        mono=mono,
    )


help_functions = {
    "capture": help_capture,
    "preview": help_preview,
}



import math
import time

from bluer_sbc.hardware.hardware import Hardware
from bluer_sbc.logger import logger


class Abstract_Hat(Hardware):
    def __init__(self):
        super().__init__()

        self.switch_pin = -1

        self.green_switch_pin = -1
        self.red_switch_pin = -1
        self.trigger_pin = -1

        self.looper_pin = -1  # red led
        self.incoming_pin = -1  # yellow led
        self.data_pin = -1  # green led
        self.outgoing_pin = -1  # blue led

        self.green_led_pin = -1
        self.red_led_pin = -1

        self.pin_history = {}

    def activated(self, pin):
        """
        is pin activated?
        :param pin: pin number
        :return: True / False
        """
        if pin in self.input_pins:
            return self.input(pin) == False

        return False

    def input(self, pin):
        """
        read pin input.
        :param pin: pin number
        :return: True / False
        """
        return True

    @property
    def input_pins(self):
        return [
            pin
            for pin in [
                self.switch_pin,
                self.green_switch_pin,
                self.red_switch_pin,
                self.trigger_pin,
            ]
            if pin != -1
        ]

    def output(self, pin, output):
        """
        set pin to output
        :param pin: pin number
        :param output: True / False
        :return: self
        """
        return self

    @property
    def output_pins(self):
        return [
            pin
            for pin in [
                self.looper_pin,
                self.incoming_pin,
                self.data_pin,
                self.outgoing_pin,
                self.green_led_pin,
                self.red_led_pin,
            ]
            if pin != -1
        ]

    def pulse(self, pin=None, frequency=None):
        """
        pulse pin.
        :param pin: pin number / "data" / "incoming" / "loop" / "outputs"
        :param frequency: frequency
        :return: self
        """
        super().pulse(pin, frequency)

        if pin == "data":
            pin = self.data_pin

        if pin == "incoming":
            pin = self.incoming_pin

        if pin == "loop":
            pin = self.looper_pin

        if pin == "outputs":
            for index, pin in enumerate(self.output_pins):
                self.pulse(pin, index)

            return self

        if pin is None:
            for pin in self.output_pins:
                self.pulse(pin, frequency)

            return self

        self.pin_history[pin] = (
            not bool(self.pin_history.get(pin, False))
            if frequency is None
            else (lambda x: x - math.floor(x))(time.time() * (10 + frequency)) >= 0.5
        )

        return self.output(pin, self.pin_history[pin])

    def release(self):
        """
        release self
        :return: self
        """
        return self

    def setup(self, pin, what, pull_up_down=None):
        """
        Set up pin.
        :param pin: pin number
        :param what: "input" / "output"
        :return: self
        """
        return self



from typing import List

from bluer_ai.host import signature as bluer_ai_signature

from bluer_sbc import fullname


def signature() -> List[str]:
    return [
        fullname(),
    ] + bluer_ai_signature()



from bluer_objects import README

design_template = "https://github.com/kamangir/blue-bracket/blob/main/designs/{}.md"

image_template = "https://github.com/kamangir/blue-bracket/raw/main/images/{}"

items = README.Items(
    [
        {
            "name": item["name"],
            "marquee": image_template.format(item["image"]),
            "url": design_template.format(item["name"]),
        }
        for item in [
            {
                "image": "blue3-1.jpg",
                "name": "blue3",
            },
            {
                "image": "chenar-grove-1.jpg",
                "name": "chenar-grove",
            },
            {
                "image": "cube-1.jpg",
                "name": "cube",
            },
            {
                "image": "eye_nano-1.jpg",
                "name": "eye_nano",
            },
        ]
    ]
)



from typing import List

from bluer_options.terminal import show_usage, xtra


def help_fake_display(
    tokens: List[str],
    mono: bool,
) -> str:
    options = xtra("dryrun", mono=mono)

    return show_usage(
        [
            "@rpi",
            "fake_display",
            f"[{options}]",
        ],
        "fake the display on an rpi.",
        mono=mono,
    )


help_functions = {
    "fake_display": help_fake_display,
}
